@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity X display handler */
 
 /* 
@@ -62,7 +64,7 @@ typedef struct MetaEdgeResistanceData MetaEdgeResistanceData;
 
 typedef void (* MetaWindowPingFunc) (MetaDisplay *display,
 				     Window       xwindow,
-				     Time         timestamp,
+				     guint32      timestamp,
 				     gpointer     user_data);
 
 
@@ -194,10 +196,10 @@ struct _MetaDisplay
   MetaWindow *expected_focus_window;
 
   /* last timestamp passed to XSetInputFocus */
-  Time last_focus_time;
+  guint32 last_focus_time;
 
   /* last user interaction time in any app */
-  Time last_user_time;
+  guint32 last_user_time;
 
   /* whether we're using mousenav (only relevant for sloppy&mouse focus modes;
    * !mouse_mode means "keynav mode")
@@ -262,6 +264,7 @@ struct _MetaDisplay
   guint       grab_have_keyboard : 1;
   guint       grab_wireframe_active : 1;
   guint       grab_was_cancelled : 1;    /* Only used in wireframe mode */
+  guint       grab_frame_action : 1;
   MetaRectangle grab_wireframe_rect;
   MetaRectangle grab_wireframe_last_xor_rect;
   MetaRectangle grab_initial_window_pos;
@@ -269,7 +272,7 @@ struct _MetaDisplay
   gboolean    grab_threshold_movement_reached; /* raise_on_click == FALSE.    */
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
-  Time        grab_motion_notify_time;
+  guint32     grab_motion_notify_time;
   int         grab_wireframe_last_display_width;
   int         grab_wireframe_last_display_height;
   GList*      grab_old_window_stacking;
@@ -283,7 +286,7 @@ struct _MetaDisplay
 
 #ifdef HAVE_XKB
   int         xkb_base_event_type;
-  Time        last_bell_time;
+  guint32     last_bell_time;
 #endif
 #ifdef HAVE_XSYNC
   /* alarm monitoring client's _NET_WM_SYNC_REQUEST_COUNTER */
@@ -311,6 +314,9 @@ struct _MetaDisplay
   /* Xinerama cache */
   unsigned int xinerama_cache_invalidated : 1;
 
+  /* Opening the display */
+  unsigned int display_opening : 1;
+
   /* Closing down the display */
   int closing;
 
@@ -388,7 +394,8 @@ struct _MetaDisplay
   )
 
 gboolean      meta_display_open                (void);
-void          meta_display_close               (MetaDisplay *display);
+void          meta_display_close               (MetaDisplay *display,
+                                                guint32      timestamp);
 MetaScreen*   meta_display_screen_for_root     (MetaDisplay *display,
                                                 Window       xroot);
 MetaScreen*   meta_display_screen_for_x_screen (MetaDisplay *display,
@@ -399,10 +406,12 @@ void          meta_display_grab                (MetaDisplay *display);
 void          meta_display_ungrab              (MetaDisplay *display);
 
 void          meta_display_unmanage_screen     (MetaDisplay *display,
-                                                MetaScreen  *screen);
+                                                MetaScreen  *screen,
+                                                guint32      timestamp);
 
 void          meta_display_unmanage_windows_for_screen (MetaDisplay *display,
-                                                        MetaScreen  *screen);
+                                                        MetaScreen  *screen,
+                                                        guint32      timestamp);
 
 /* A given MetaWindow may have various X windows that "belong"
  * to it, such as the frame window.
@@ -431,21 +440,22 @@ void     meta_display_set_grab_op_cursor (MetaDisplay *display,
                                           MetaGrabOp   op,
                                           gboolean     change_pointer,
                                           Window       grab_xwindow,
-                                          Time         timestamp);
+                                          guint32      timestamp);
 
 gboolean meta_display_begin_grab_op (MetaDisplay *display,
                                      MetaScreen  *screen,
                                      MetaWindow  *window,
                                      MetaGrabOp   op,
                                      gboolean     pointer_already_grabbed,
+                                     gboolean     frame_action,
                                      int          event_serial,
                                      int          button,
                                      gulong       modmask,
-                                     Time         timestamp,
+                                     guint32      timestamp,
                                      int          root_x,
                                      int          root_y);
 void     meta_display_end_grab_op   (MetaDisplay *display,
-                                     Time         timestamp);
+                                     guint32      timestamp);
 
 void    meta_display_check_threshold_reached (MetaDisplay *display,
                                               int          x,
@@ -482,12 +492,12 @@ void meta_display_retheme_all (void);
 void meta_display_set_cursor_theme (const char *theme, 
 				    int         size);
 
-void     meta_display_ping_window              (MetaDisplay        *display,
-						MetaWindow         *window,
-						Time                timestamp,
-						MetaWindowPingFunc  ping_reply_func,
-						MetaWindowPingFunc  ping_timeout_func,
-						void               *user_data);
+void meta_display_ping_window      (MetaDisplay        *display,
+                                    MetaWindow         *window,
+                                    guint32             timestamp,
+                                    MetaWindowPingFunc  ping_reply_func,
+                                    MetaWindowPingFunc  ping_timeout_func,
+                                    void               *user_data);
 gboolean meta_display_window_has_pending_pings (MetaDisplay        *display,
 						MetaWindow         *window);
 
@@ -546,7 +556,7 @@ gboolean meta_display_focus_sentinel_clear (MetaDisplay *display);
 void meta_display_set_input_focus_window   (MetaDisplay *display, 
                                             MetaWindow  *window,
                                             gboolean     focus_frame,
-                                            Time         timestamp);
+                                            guint32      timestamp);
 
 /* meta_display_focus_the_no_focus_window is called when the
  * designated no_focus_window should be focused, but is otherwise the
@@ -554,7 +564,7 @@ void meta_display_set_input_focus_window   (MetaDisplay *display,
  */
 void meta_display_focus_the_no_focus_window (MetaDisplay *display, 
                                              MetaScreen  *screen,
-                                             Time         timestamp);
+                                             guint32      timestamp);
 
 void meta_display_queue_autoraise_callback  (MetaDisplay *display,
                                              MetaWindow  *window);
