@@ -1,3 +1,5 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity window frame manager widget */
 
 /* 
@@ -22,6 +24,7 @@
  */
 
 #include <config.h>
+#include <math.h>
 #include "boxes.h"
 #include "frames.h"
 #include "util.h"
@@ -755,10 +758,10 @@ meta_frames_apply_shapes (MetaFrames *frames,
 
   meta_frames_calc_geometry (frames, frame, &fgeom);
 
-  if (!(fgeom.top_left_corner_rounded ||
-        fgeom.top_right_corner_rounded ||
-        fgeom.bottom_left_corner_rounded ||
-        fgeom.bottom_right_corner_rounded ||
+  if (!(fgeom.top_left_corner_rounded_radius != 0 ||
+        fgeom.top_right_corner_rounded_radius != 0 ||
+        fgeom.bottom_left_corner_rounded_radius != 0 ||
+        fgeom.bottom_right_corner_rounded_radius != 0 ||
         window_has_shape))
     {
       if (frame->shape_applied)
@@ -783,102 +786,72 @@ meta_frames_apply_shapes (MetaFrames *frames,
   
   corners_xregion = XCreateRegion ();
   
-  if (fgeom.top_left_corner_rounded)
+  if (fgeom.top_left_corner_rounded_radius != 0)
     {
-      xrect.x = 0;
-      xrect.y = 0;
-      xrect.width = 5;
-      xrect.height = 1;
-      
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      const int radius = fgeom.top_left_corner_rounded_radius;
+      int i;
 
-      xrect.y = 1;
-      xrect.width = 3;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = 2;
-      xrect.width = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-
-      xrect.y = 3;
-      xrect.width = 1;
-      xrect.height = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      for (i=0; i<radius; i++)
+        {
+          const int width = 1 + (radius - floor(sqrt(radius*radius - (radius-i)*(radius-i)) + 0.5));
+          xrect.x = 0;
+          xrect.y = i;
+          xrect.width = width;
+          xrect.height = 1;
+          
+          XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+        }
     }
 
-  if (fgeom.top_right_corner_rounded)
+  if (fgeom.top_right_corner_rounded_radius != 0)
     {
-      xrect.x = new_window_width - 5;
-      xrect.y = 0;
-      xrect.width = 5;
-      xrect.height = 1;
-      
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      const int radius = fgeom.top_right_corner_rounded_radius;
+      int i;
 
-      xrect.y = 1;
-      xrect.x = new_window_width - 3;
-      xrect.width = 3;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = 2;
-      xrect.x = new_window_width - 2;
-      xrect.width = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-
-      xrect.y = 3;
-      xrect.x = new_window_width - 1;
-      xrect.width = 1;
-      xrect.height = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      for (i=0; i<radius; i++)
+        {
+          const int width = 1 + (radius - floor(sqrt(radius*radius - (radius-i)*(radius-i)) + 0.5));
+          xrect.x = new_window_width - width;
+          xrect.y = i;
+          xrect.width = width;
+          xrect.height = 1;
+          
+          XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+        }
     }
 
-  if (fgeom.bottom_left_corner_rounded)
+  if (fgeom.bottom_left_corner_rounded_radius != 0)
     {
-      xrect.x = 0;
-      xrect.y = new_window_height - 1;
-      xrect.width = 5;
-      xrect.height = 1;
-      
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = new_window_height - 2;
-      xrect.width = 3;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = new_window_height - 3;
-      xrect.width = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      const int radius = fgeom.bottom_left_corner_rounded_radius;
+      int i;
 
-      xrect.y = new_window_height - 5;
-      xrect.width = 1;
-      xrect.height = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      for (i=0; i<radius; i++)
+        {
+          const int width = 1 + (radius - floor(sqrt(radius*radius - (radius-i)*(radius-i)) + 0.5));
+          xrect.x = 0;
+          xrect.y = new_window_height - i;
+          xrect.width = width;
+          xrect.height = 1;
+          
+          XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+        }
     }
 
-  if (fgeom.bottom_right_corner_rounded)
+  if (fgeom.bottom_right_corner_rounded_radius != 0)
     {
-      xrect.x = new_window_width - 5;
-      xrect.y = new_window_height - 1;
-      xrect.width = 5;
-      xrect.height = 1;
-      
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = new_window_height - 2;
-      xrect.x = new_window_width - 3;
-      xrect.width = 3;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
-      
-      xrect.y = new_window_height - 3;
-      xrect.x = new_window_width - 2;
-      xrect.width = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      const int radius = fgeom.bottom_right_corner_rounded_radius;
+      int i;
 
-      xrect.y = new_window_height - 5;
-      xrect.x = new_window_width - 1;
-      xrect.width = 1;
-      xrect.height = 2;
-      XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+      for (i=0; i<radius; i++)
+        {
+          const int width = 1 + (radius - floor(sqrt(radius*radius - (radius-i)*(radius-i)) + 0.5));
+          xrect.x = new_window_width - width;
+          xrect.y = new_window_height - i;
+          xrect.width = width;
+          xrect.height = 1;
+          
+          XUnionRectWithRegion (&xrect, corners_xregion, corners_xregion);
+        }
     }
   
   window_xregion = XCreateRegion ();
@@ -1100,6 +1073,24 @@ show_tip_now (MetaFrames *frames)
     case META_FRAME_CONTROL_UNMAXIMIZE:
       tiptext = _("Unmaximize Window");
       break;
+    case META_FRAME_CONTROL_SHADE:
+      tiptext = _("Roll Up Window");
+      break;
+    case META_FRAME_CONTROL_UNSHADE:
+      tiptext = _("Unroll Window");
+      break;
+    case META_FRAME_CONTROL_ABOVE:
+      tiptext = _("Keep Window On Top");
+      break;
+    case META_FRAME_CONTROL_UNABOVE:
+      tiptext = _("Remove Window From Top");
+      break;
+    case META_FRAME_CONTROL_STICK:
+      tiptext = _("Always On Visible Workspace");
+      break;
+    case META_FRAME_CONTROL_UNSTICK:
+      tiptext = _("Put Window On Only One Workspace");
+      break;
     case META_FRAME_CONTROL_RESIZE_SE:
       break;
     case META_FRAME_CONTROL_RESIZE_S:
@@ -1306,6 +1297,12 @@ meta_frames_button_press_event (GtkWidget      *widget,
        control == META_FRAME_CONTROL_UNMAXIMIZE ||
        control == META_FRAME_CONTROL_MINIMIZE ||
        control == META_FRAME_CONTROL_DELETE ||
+       control == META_FRAME_CONTROL_SHADE ||
+       control == META_FRAME_CONTROL_UNSHADE ||
+       control == META_FRAME_CONTROL_ABOVE ||
+       control == META_FRAME_CONTROL_UNABOVE ||
+       control == META_FRAME_CONTROL_STICK ||
+       control == META_FRAME_CONTROL_UNSTICK ||
        control == META_FRAME_CONTROL_MENU))
     {
       MetaGrabOp op = META_GRAB_OP_NONE;
@@ -1327,6 +1324,24 @@ meta_frames_button_press_event (GtkWidget      *widget,
         case META_FRAME_CONTROL_MENU:
           op = META_GRAB_OP_CLICKING_MENU;
           break;
+        case META_FRAME_CONTROL_SHADE:
+          op = META_GRAB_OP_CLICKING_SHADE;
+          break;
+        case META_FRAME_CONTROL_UNSHADE:
+          op = META_GRAB_OP_CLICKING_UNSHADE;
+          break;
+        case META_FRAME_CONTROL_ABOVE:
+          op = META_GRAB_OP_CLICKING_ABOVE;
+          break;
+        case META_FRAME_CONTROL_UNABOVE:
+          op = META_GRAB_OP_CLICKING_UNABOVE;
+          break;
+        case META_FRAME_CONTROL_STICK:
+          op = META_GRAB_OP_CLICKING_STICK;
+          break;
+        case META_FRAME_CONTROL_UNSTICK:
+          op = META_GRAB_OP_CLICKING_UNSTICK;
+          break;
         default:
           g_assert_not_reached ();
           break;
@@ -1336,6 +1351,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
                                frame->xwindow,
                                op,
                                TRUE,
+                               TRUE,
                                meta_ui_get_last_event_serial (gdk_display),
                                event->button,
                                0,
@@ -1426,6 +1442,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
                                  frame->xwindow,
                                  op,
                                  TRUE,
+                                 TRUE,
                                  meta_ui_get_last_event_serial (gdk_display),
                                  event->button,
                                  0,
@@ -1446,6 +1463,7 @@ meta_frames_button_press_event (GtkWidget      *widget,
                                    frame->xwindow,
                                    META_GRAB_OP_MOVING,
                                    TRUE,
+                                   TRUE,
                                    meta_ui_get_last_event_serial (gdk_display),
                                    event->button,
                                    0,
@@ -1565,6 +1583,48 @@ meta_frames_button_release_event    (GtkWidget           *widget,
           meta_core_end_grab_op (gdk_display, event->time);
           break;
 
+        case META_GRAB_OP_CLICKING_SHADE:
+          if (control == META_FRAME_CONTROL_SHADE)
+            meta_core_shade (gdk_display, frame->xwindow, event->time);
+          
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+ 
+        case META_GRAB_OP_CLICKING_UNSHADE:
+          if (control == META_FRAME_CONTROL_UNSHADE)
+            meta_core_unshade (gdk_display, frame->xwindow, event->time);
+
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+
+        case META_GRAB_OP_CLICKING_ABOVE:
+          if (control == META_FRAME_CONTROL_ABOVE)
+            meta_core_make_above (gdk_display, frame->xwindow);
+          
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+ 
+        case META_GRAB_OP_CLICKING_UNABOVE:
+          if (control == META_FRAME_CONTROL_UNABOVE)
+            meta_core_unmake_above (gdk_display, frame->xwindow);
+
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+
+        case META_GRAB_OP_CLICKING_STICK:
+          if (control == META_FRAME_CONTROL_STICK)
+            meta_core_stick (gdk_display, frame->xwindow);
+
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+ 
+        case META_GRAB_OP_CLICKING_UNSTICK:
+          if (control == META_FRAME_CONTROL_UNSTICK)
+            meta_core_unstick (gdk_display, frame->xwindow);
+
+          meta_core_end_grab_op (gdk_display, event->time);
+          break;
+          
         default:
           break;
         }
@@ -1588,6 +1648,7 @@ meta_frames_update_prelit_control (MetaFrames      *frames,
   MetaFrameControl old_control;
   MetaCursor cursor;
 
+
   meta_verbose ("Updating prelit control from %u to %u\n",
                 frame->prelit_control, control);
   
@@ -1611,6 +1672,18 @@ meta_frames_update_prelit_control (MetaFrames      *frames,
       break;
     case META_FRAME_CONTROL_UNMAXIMIZE:
       break;
+    case META_FRAME_CONTROL_SHADE:
+      break;
+    case META_FRAME_CONTROL_UNSHADE:
+      break;
+    case META_FRAME_CONTROL_ABOVE:
+      break;
+    case META_FRAME_CONTROL_UNABOVE:
+      break;
+    case META_FRAME_CONTROL_STICK:
+      break;
+    case META_FRAME_CONTROL_UNSTICK:
+      break;
     case META_FRAME_CONTROL_RESIZE_SE:
       cursor = META_CURSOR_SE_RESIZE;
       break;
@@ -1648,6 +1721,12 @@ meta_frames_update_prelit_control (MetaFrames      *frames,
     case META_FRAME_CONTROL_MINIMIZE:
     case META_FRAME_CONTROL_MAXIMIZE:
     case META_FRAME_CONTROL_DELETE:
+    case META_FRAME_CONTROL_SHADE:
+    case META_FRAME_CONTROL_UNSHADE:
+    case META_FRAME_CONTROL_ABOVE:
+    case META_FRAME_CONTROL_UNABOVE:
+    case META_FRAME_CONTROL_STICK:
+    case META_FRAME_CONTROL_UNSTICK:
     case META_FRAME_CONTROL_UNMAXIMIZE:
       /* leave control set */
       break;
@@ -1696,6 +1775,12 @@ meta_frames_motion_notify_event     (GtkWidget           *widget,
     case META_GRAB_OP_CLICKING_MINIMIZE:
     case META_GRAB_OP_CLICKING_MAXIMIZE:
     case META_GRAB_OP_CLICKING_UNMAXIMIZE:
+    case META_GRAB_OP_CLICKING_SHADE:
+    case META_GRAB_OP_CLICKING_UNSHADE:
+    case META_GRAB_OP_CLICKING_ABOVE:
+    case META_GRAB_OP_CLICKING_UNABOVE:
+    case META_GRAB_OP_CLICKING_STICK:
+    case META_GRAB_OP_CLICKING_UNSTICK:
       {
         MetaFrameControl control;
         int x, y;
@@ -1714,8 +1799,20 @@ meta_frames_motion_notify_event     (GtkWidget           *widget,
                 grab_op == META_GRAB_OP_CLICKING_MINIMIZE) ||
                (control == META_FRAME_CONTROL_MAXIMIZE &&
                 (grab_op == META_GRAB_OP_CLICKING_MAXIMIZE ||
-                 grab_op == META_GRAB_OP_CLICKING_UNMAXIMIZE))))
-          control = META_FRAME_CONTROL_NONE;
+                 grab_op == META_GRAB_OP_CLICKING_UNMAXIMIZE)) ||
+               (control == META_FRAME_CONTROL_SHADE &&
+                grab_op == META_GRAB_OP_CLICKING_SHADE) ||
+               (control == META_FRAME_CONTROL_UNSHADE &&
+                grab_op == META_GRAB_OP_CLICKING_UNSHADE) ||
+               (control == META_FRAME_CONTROL_ABOVE &&
+                grab_op == META_GRAB_OP_CLICKING_ABOVE) ||
+               (control == META_FRAME_CONTROL_UNABOVE &&
+                grab_op == META_GRAB_OP_CLICKING_UNABOVE) ||
+               (control == META_FRAME_CONTROL_STICK &&
+                grab_op == META_GRAB_OP_CLICKING_STICK) ||
+               (control == META_FRAME_CONTROL_UNSTICK &&
+                grab_op == META_GRAB_OP_CLICKING_UNSTICK)))
+           control = META_FRAME_CONTROL_NONE;
         
         /* Update prelit control and cursor */
 	meta_frames_update_prelit_control (frames, frame, control);
@@ -2095,6 +2192,42 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
       else
         button_states[META_BUTTON_TYPE_MAXIMIZE] = META_BUTTON_STATE_PRELIGHT;
       break;
+    case META_FRAME_CONTROL_SHADE:
+      if (grab_op == META_GRAB_OP_CLICKING_SHADE)
+        button_states[META_BUTTON_TYPE_SHADE] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_SHADE] = META_BUTTON_STATE_PRELIGHT;
+      break;
+    case META_FRAME_CONTROL_UNSHADE:
+      if (grab_op == META_GRAB_OP_CLICKING_UNSHADE)
+        button_states[META_BUTTON_TYPE_UNSHADE] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_UNSHADE] = META_BUTTON_STATE_PRELIGHT;
+      break;
+    case META_FRAME_CONTROL_ABOVE:
+      if (grab_op == META_GRAB_OP_CLICKING_ABOVE)
+        button_states[META_BUTTON_TYPE_ABOVE] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_ABOVE] = META_BUTTON_STATE_PRELIGHT;
+      break;
+    case META_FRAME_CONTROL_UNABOVE:
+      if (grab_op == META_GRAB_OP_CLICKING_UNABOVE)
+        button_states[META_BUTTON_TYPE_UNABOVE] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_UNABOVE] = META_BUTTON_STATE_PRELIGHT;
+      break;
+    case META_FRAME_CONTROL_STICK:
+      if (grab_op == META_GRAB_OP_CLICKING_STICK)
+        button_states[META_BUTTON_TYPE_STICK] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_STICK] = META_BUTTON_STATE_PRELIGHT;
+      break;
+    case META_FRAME_CONTROL_UNSTICK:
+      if (grab_op == META_GRAB_OP_CLICKING_UNSTICK)
+        button_states[META_BUTTON_TYPE_UNSTICK] = META_BUTTON_STATE_PRESSED;
+      else
+        button_states[META_BUTTON_TYPE_UNSTICK] = META_BUTTON_STATE_PRELIGHT;
+      break;
     case META_FRAME_CONTROL_DELETE:
       if (grab_op == META_GRAB_OP_CLICKING_DELETE)
         button_states[META_BUTTON_TYPE_CLOSE] = META_BUTTON_STATE_PRESSED;
@@ -2180,17 +2313,52 @@ static void
 meta_frames_set_window_background (MetaFrames   *frames,
                                    MetaUIFrame  *frame)
 {
-  gtk_style_set_background (GTK_WIDGET (frames)->style,
-                            frame->window, GTK_STATE_NORMAL);
+  MetaFrameFlags flags;
+  MetaFrameType type;
+  MetaFrameStyle *style;
+  gboolean frame_exists;
 
-#if 0
-  /* This is what we want for transparent background */
-  {
-    col.pixel = 0;
-    gdk_window_set_background (window, &col);
-  }
-#endif
-}
+  frame_exists = meta_core_window_has_frame (gdk_display, frame->xwindow);
+
+  if (frame_exists)
+    {
+      flags = meta_core_get_frame_flags (gdk_display, frame->xwindow);
+      type = meta_core_get_frame_type (gdk_display, frame->xwindow);
+      style = meta_theme_get_frame_style (meta_theme_get_current (),
+                                          type, flags);
+    }
+
+  if (frame_exists && style->window_background_color != NULL)
+    {
+      GdkColor color;
+      GdkVisual *visual;
+
+      meta_color_spec_render (style->window_background_color,
+                              GTK_WIDGET (frames),
+                              &color);
+
+      /* Fill in color.pixel */
+
+      gdk_rgb_find_color (gtk_widget_get_colormap (GTK_WIDGET (frames)),
+                          &color);
+
+      /* Set A in ARGB to window_background_alpha, if we have ARGB */
+
+      visual = gtk_widget_get_visual (GTK_WIDGET (frames));
+      if (visual->depth == 32) /* we have ARGB */
+        {
+          color.pixel = (color.pixel & 0xffffff) &
+            style->window_background_alpha << 24;
+        }
+
+      gdk_window_set_background (frame->window, &color);
+    }
+  else
+    {
+      gtk_style_set_background (GTK_WIDGET (frames)->style,
+                                frame->window, GTK_STATE_NORMAL);
+    }
+ }
 
 static gboolean
 meta_frames_enter_notify_event      (GtkWidget           *widget,
@@ -2257,6 +2425,24 @@ control_rect (MetaFrameControl control,
     case META_FRAME_CONTROL_UNMAXIMIZE:
       rect = &fgeom->max_rect.visible;
       break;
+    case META_FRAME_CONTROL_SHADE:
+      rect = &fgeom->shade_rect.visible;
+      break;
+    case META_FRAME_CONTROL_UNSHADE:
+      rect = &fgeom->unshade_rect.visible;
+      break;
+    case META_FRAME_CONTROL_ABOVE:
+      rect = &fgeom->above_rect.visible;
+      break;
+    case META_FRAME_CONTROL_UNABOVE:
+      rect = &fgeom->unabove_rect.visible;
+      break;
+    case META_FRAME_CONTROL_STICK:
+      rect = &fgeom->stick_rect.visible;
+      break;
+    case META_FRAME_CONTROL_UNSTICK:
+      rect = &fgeom->unstick_rect.visible;
+      break;
     case META_FRAME_CONTROL_RESIZE_SE:
       break;
     case META_FRAME_CONTROL_RESIZE_S:
@@ -2334,6 +2520,36 @@ get_control (MetaFrames *frames,
         return META_FRAME_CONTROL_MAXIMIZE;
     }
       
+  if (POINT_IN_RECT (x, y, fgeom.shade_rect.clickable))
+    {
+      return META_FRAME_CONTROL_SHADE;
+    }
+
+  if (POINT_IN_RECT (x, y, fgeom.unshade_rect.clickable))
+    {
+      return META_FRAME_CONTROL_UNSHADE;
+    }
+
+  if (POINT_IN_RECT (x, y, fgeom.above_rect.clickable))
+    {
+      return META_FRAME_CONTROL_ABOVE;
+    }
+
+  if (POINT_IN_RECT (x, y, fgeom.unabove_rect.clickable))
+    {
+      return META_FRAME_CONTROL_UNABOVE;
+    }
+
+  if (POINT_IN_RECT (x, y, fgeom.stick_rect.clickable))
+    {
+      return META_FRAME_CONTROL_STICK;
+    }
+
+  if (POINT_IN_RECT (x, y, fgeom.unstick_rect.clickable))
+    {
+      return META_FRAME_CONTROL_UNSTICK;
+    }
+
   /* South resize always has priority over north resize,
    * in case of overlap.
    */
